package com.example.tomcat;

import org.apache.catalina.session.StandardManager;
import org.apache.catalina.Container;
import org.apache.catalina.Manager;
import org.apache.catalina.Session;

import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * This class implements a session manager that maintains a
 * single, shared session object (for each user) for all webapps.
 * 
 * This class is a Decorator on the StandardManager.
 * 
 * @author bbasham
 */
public class SharedSessionManager implements Manager {

    private StandardManager delegate;
    
    public SharedSessionManager()
    {
	delegate = new StandardManager();
    }
    
    //
    // The primary Manager methods
    //

    public Session createSession(String sessionId)
    {
	Session session = getGlobalSession();
	if ( session == null )
	{
	    session = delegate.createSession(sessionId);
	}
	System.out.println(String.format("%s  createSession( %s ) = %s", this, sessionId, session));
	return session;
    }

    public Session findSession(String id) throws IOException
    {
	Session session = getGlobalSession();
	if ( session == null )
	{
	    session = delegate.findSession(id);
	}
	System.out.println(String.format("%s  findSession( %s ) = %s", this, id, session));
	return session;
    }
    
    //
    // Global ID management code
    //

    /**
     * This package-private method is used by the
     * {@link com.example.tomcat.SharedSessionRequestValve} class
     * to keep track of the GLOBALID for a given request.
     */
    static synchronized boolean setGlobalIdForRequest(String globalID)
    {
	boolean result = false;
	Session session = GLOBALID_TO_SESSION.get(globalID);
	if ( session != null && session.isValid() )
	{
	    GLOBAL_ID.set(globalID);
	    result = true;
	}
	return result;
    }

    /**
     * 
     */
    static synchronized void storeGlobalSession(String globalID, Session session)
    {
	GLOBALID_TO_SESSION.put(globalID, session);
    }
    
    /**
     * 
     * @return
     */
    private static synchronized Session getGlobalSession()
    {
	String globalID = GLOBAL_ID.get();
	if ( globalID != null )
	{
	    return GLOBALID_TO_SESSION.get(globalID);
	}
	return null;
    }
    
    private static final Map<String, Session> GLOBALID_TO_SESSION = new HashMap<String, Session>();
    private static ThreadLocal<String> GLOBAL_ID = new ThreadLocal<String>()
    {
	String value;
        public String get()
        {
            return value;
        }
        public void set(String value)
        {
            this.value = value;
        }
        public void remove()
        {
            this.value = null;
        }
    };
    
    //
    // The rest of the Manager methods (pass-through delegation)
    //

    public void add(Session session) {
	delegate.add(session);
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
	delegate.addPropertyChangeListener(listener);
    }

    public void backgroundProcess() {
	delegate.backgroundProcess();
    }

    public Session createEmptySession() {
	Session session = delegate.createEmptySession();
	return session;
    }

    @Deprecated
    public Session createSession() {
	Session session = delegate.createSession();
	return session;
    }

    public Session[] findSessions() {
	return delegate.findSessions();
    }

    public int getActiveSessions() {
	return delegate.getActiveSessions();
    }

    public Container getContainer() {
	return delegate.getContainer();
    }

    public boolean getDistributable() {
	return delegate.getDistributable();
    }

    public int getExpiredSessions() {
	return delegate.getExpiredSessions();
    }

    public String getInfo() {
	return delegate.getInfo();
    }

    public int getMaxActive() {
	return delegate.getMaxActive();
    }

    public int getMaxInactiveInterval() {
	return delegate.getMaxInactiveInterval();
    }

    public int getRejectedSessions() {
	return delegate.getRejectedSessions();
    }

    public int getSessionAverageAliveTime() {
	return delegate.getSessionAverageAliveTime();
    }

    public int getSessionCounter() {
	return delegate.getSessionCounter();
    }

    public int getSessionIdLength() {
	return delegate.getSessionIdLength();
    }

    public int getSessionMaxAliveTime() {
	return delegate.getSessionMaxAliveTime();
    }

    public void load() throws ClassNotFoundException, IOException {
	delegate.load();
    }

    public void remove(Session session) {
	delegate.remove(session);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
	delegate.removePropertyChangeListener(listener);
    }

    public void setContainer(Container container) {
	delegate.setContainer(container);
    }

    public void setDistributable(boolean distributable) {
	delegate.setDistributable(distributable);
    }

    public void setExpiredSessions(int expiredSessions) {
	delegate.setExpiredSessions(expiredSessions);
    }

    public void setMaxActive(int maxActive) {
	delegate.setMaxActive(maxActive);
    }

    public void setMaxInactiveInterval(int interval) {
	delegate.setMaxInactiveInterval(interval);
    }

    public void setRejectedSessions(int rejectedSessions) {
	delegate.setRejectedSessions(rejectedSessions);
    }

    public void setSessionAverageAliveTime(int sessionAverageAliveTime) {
	delegate.setSessionAverageAliveTime(sessionAverageAliveTime);
    }

    public void setSessionCounter(int sessionCounter) {
	delegate.setSessionCounter(sessionCounter);
    }

    public void setSessionIdLength(int idLength) {
	delegate.setSessionIdLength(idLength);
    }

    public void setSessionMaxAliveTime(int sessionMaxAliveTime) {
	delegate.setSessionMaxAliveTime(sessionMaxAliveTime);
    }

    public void unload() throws IOException {
	delegate.unload();
    }

}
